package edu.gmu.cs475;


import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;

public abstract class AbstractKeyValueServer implements IKeyValueServer {
    private final HashMap<String, Object> map = new HashMap<>();

    /**
     * Retrieve an item from the underlying store - you must call this from your KeyValueStore
     *
     * @param key key to retrieve
     * @return The value stored at the given key, or null if none exists
     * @throws NullPointerException if key is null
     */
    protected Object _get(String key) {
        if (key == null)
            throw new NullPointerException();
        synchronized (map) {
            return map.get(key);
        }
    }

    /**
     * Add an item to the underlying store - you must call this from your KeyValueStore
     *
     * @param key   key to set
     * @param value value to store
     * @throws NullPointerException if key or value is null
     */
    protected void _set(String key, Object value) {
        if (key == null || value == null) throw new NullPointerException();
        synchronized (map) {
            map.put(key, value);
        }
    }

    /**
     * Remove an item from the underlying store - you must call this from your KeyValueStore
     *
     * @param key key to remove
     * @return true if the value was removed, false if not
     * @throws NullPointerException if key is null
     */
    protected boolean _remove(String key) {
        if (key == null)
            throw new NullPointerException();
        synchronized (map) {
            return map.remove(key) != null;
        }
    }

    /**
     * Enumerates all of the keys currently in the map
     *
     * @return Set containing all currently valid keys
     */
    protected Set<String> _listKeys() {
        synchronized (map) {
            return new HashSet<>(map.keySet());
        }
    }

    @Override
    public String toString() {
        return "KeyValueStore{" +
                "map=" + map +
                '}';
    }

}
